# -*- coding: utf-8 -*-
# vim: shiftwidth=4 tabstop=4 expandtab softtabstop=4 ai

# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function


from django.core.management.base import BaseCommand

from medlemssys.medlem.models import update_denormalized_fields
from medlemssys.statistikk.views import send_overvakingar
from medlemssys.statistikk.views import update_lokallagstat


class Command(BaseCommand):
    help = "Do nightly chores"

    def handle(self, *args, **options):
        update_denormalized_fields()
        update_lokallagstat()
        send_overvakingar()



# -*- coding: utf-8 -*-
# vim: shiftwidth=4 tabstop=4 expandtab softtabstop=4 ai

# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import print_function

import os
from optparse import make_option

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError

from medlemssys.innhenting.management import nmu
from medlemssys.medlem.models import update_denormalized_fields
from medlemssys.statistikk.views import send_overvakingar
from medlemssys.statistikk.views import update_lokallagstat


class Command(BaseCommand):
    args = '[ medlem.csv [ lokallag.csv [ betaling.csv ] ] ]'
    help = "Importerer medlemane inn i databasen"
    force_update = False
    option_list = BaseCommand.option_list + (
        make_option('-f', '--force-update',
            action='store_true',
            dest='force_update',
            default=False,
            help="tving gjennom oppdatering av giroar"),
        make_option('--importer',
            dest='importer',
            default='guess_csv',
            help="importeringssystem (guess_csv, nmu_access eller nmu_mamut)"),
        )

    def handle(self, *args, **options):
        if options['force_update']:
            self.force_update = True


        def getarg(num):
            try:
                return args[num]
            except:
                return None
        medlem_csv = getarg(0)
        lag_csv = getarg(1)
        bet_csv = getarg(2)

        if options['importer'] == 'guess_csv':
            imp = nmu.GuessingCSVImporter()
        elif options['importer'] == 'nmu_access':
            imp = nmu.AccessImporter()
        elif options['importer'] == 'nmu_mamut':
            imp = nmu.MamutImporter()
        else:
            raise CommandError("Importeren finst ikkje ({0})".format(options['importer']).encode('utf8'))

        if lag_csv:
            for i in imp.import_lag(lag_csv).values():
                self.stdout.write(u"Lag: {0}\n".format(i))

        if medlem_csv:
            for i in imp.import_medlem(medlem_csv):
                self.stdout.write(u"Medlem: {0}\n".format(i))

        if bet_csv:
            for i in imp.import_bet(bet_csv, force_update=self.force_update):
                self.stdout.write(u"Betaling: {0}\n".format(i))

        update_denormalized_fields()
        update_lokallagstat()
        send_overvakingar()



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
import datetime
import logging
from django.db.models import Q

from medlemssys.medlem.models import Giro
from medlemssys.medlem.models import update_denormalized_fields

from . import mod10

logger = logging.getLogger(__name__)

class OCRError(Exception):
    pass


class OCR(object):
    TRANSAKSJONSTYPE = {
        "10": "Giro belasta konto",
        "11": "Faste oppdrag",
        "12": "Direkte remittering",
        "13": "Bedriftsterminalgiro (BTG)",
        "14": "Skrankegiro",
        "15": "AvtaleGiro",
        "16": "TeleGiro",
        "17": "Giro - betalt kontant",
        "18": "Nettgiro - reversering m KID",
        "19": "Nettgiro - kjøp m KID",
        "20": "Nettgiro - reversering m fritekst",
        "21": "Nettgiro - kjøp m fritekst",
    }

    def __init__(self):
        self.data = []

    def from_filename(self, path):
        fp = open(path)
        self._parse(fp)

    def from_string(self, string):
        self._parse(iter(string.splitlines()))

    def process_to_db(self):
        for f in self.data:
            f['msg'] = []
            f['giro'] = None
            f['processed'] = False
            try:
                giro = Giro.objects.get(
                        Q(oppretta__year=f['dato'].year) | Q(oppretta__year=(f['dato'].year - 1)),
                        kid=f['kid'])
            except Giro.DoesNotExist:
                msg = "Fann ikkje giroen, {dato:6s} {belop:4n} {kid:12s} {transaksjon}".format(**f)
                logger.warning(msg)
                f['msg'].append(msg)
                continue
            f['giro'] = giro
            if giro.betalt():
                msg = "Giroen ({0}) er allereie betalt".format(giro.admin_change())
                f['msg'].append(msg)
                logger.warning("{giro.medlem}: {msg}".format(giro=giro, msg=msg))
                continue

            if f['belop'] < giro.belop:
                msg = "{giro}: for lite betalt! Rekna {giro.belop}, fekk {belop} ({giro.pk})".format(giro=giro, belop=f['belop'])
                logger.warning("{giro.medlem}: {msg}".format(giro=giro, msg=msg))
                f['msg'].append(msg)
            elif f['belop'] > giro.belop:
                # XXX: Splitt opp? Registrer ein donasjon?
                msg = "{giro}: Betalte meir, venta {giro.belop}, fekk {belop} ({giro.pk})".format(giro=giro, belop=f['belop'])
                logger.info("{giro.medlem}: {msg}".format(giro=giro, msg=msg))
                f['msg'].append(msg)
                giro.innbetalt = f['dato']
            else:
                giro.innbetalt = f['dato']
                #self.err("{belop:3n}kr ({giro.pk}) {giro} ".format(giro=giro, belop=f['belop']))

            giro.innbetalt_belop = f['belop']
            giro.konto = 'A' # Medlemskonto (KID)
            if giro.desc:
                giro.desc += '\nOCR'
            else:
                giro.desc = 'OCR'
            giro.save()
            f['processed'] = True

            if giro.medlem.status == 'I' and giro.innbetalt_belop >= giro.belop:
                f['msg'].append("{medlem} var infoperson, flyttar til medlem etter betaling".format(medlem=giro.medlem))
                giro.medlem.status = 'M'
                giro.medlem.save()

        update_denormalized_fields()


    def _parse(self, ocr_data):
        for row in ocr_data:
            row = row.strip()
            if row[0:8] == "NY000010":
                # Startrecord for sending
                #forsendelsesnr = row[16:23]
                break
        else:
            raise OCRError('Fann ikkje startrecord for sending')

        for row in ocr_data:
            row = row.strip()

            if row[0:6] == "NY0900":
                if row[6:8] == "20":
                    # Startrecord for oppdrag
                    pass
                elif row[6:8] == "88":
                    # Sluttrecord for oppdrag
                    pass
                else:
                    raise OCRError("Forstår ikkje kode {0} ({1})!".format(row[6:8], row))
                continue

            elif row[0:4] == "NY09":
                if row[6:8] != "30":
                    raise OCRError("Venta startrecord 30, fekk {0}. ({1})!".format(row[6:8], row))

                # Transaksjon
                trans = self.TRANSAKSJONSTYPE[row[4:6]]
                dato = row[15:21]
                dato = datetime.date(2000 + int(dato[4:6]), int(dato[2:4]), int(dato[0:2]))
                belop = int(row[32:49])/100.0
                kid = row[49:74].strip()
                if not mod10.check_number(kid.strip()):
                    raise OCRError("KID-nummer validerte ikkje ({0})".format(kid))

                row2 = ocr_data.next()
                oppdr_dato = row2[41:47]
                fra_konto = row2[47:58]

                fritekst = ""
                if row[4:6] == "20" or row[4:6] == "21":
                    row3 = ocr_data.next()
                    fritekst = row3[15:55]

                self.data.append(dict(
                            transaksjon=trans,
                            dato=dato,
                            belop=belop,
                            kid=kid,
                            oppdragsdato=oppdr_dato,
                            fra_konto=fra_konto,
                            fritekst=fritekst,
                        ))
        return self.data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
import sys, random

def check_number(digits):
    _sum = 0
    alt = False
    for d in reversed(str(digits)):
        d = int(d)
        assert 0 <= d <= 9
        if alt:
            d *= 2
            if d > 9:
                d -= 9
        _sum += d
        alt = not alt
    return (_sum % 10) == 0

def mod10(kid):
    _sum = 0
    alt = True
    for s in reversed(str(kid)):
        d = int(s)
        assert 0 <= d <= 9
        if alt:
            d *= 2
            if d > 9:
                d -= 9
        _sum += d
        alt = not alt

    controlbit = 10 - (_sum % 10)
    if controlbit == 10:
        controlbit = 0

    return controlbit

def kid_add_controlbit(kid):
    new_kid = str((int(kid) * 10) + mod10(kid))
    new_kid = new_kid.zfill(len(kid)+1)
    return new_kid


if __name__ == '__main__':
    kid = str(234567)
    for i in range(10):
        nkid = kid_add_controlbit(kid)
        correct = check_number(nkid)

        print "%s\t%s\t(fra %s, nr %3d)" % (nkid, correct, kid, i)

        if (not correct):
            sys.exit()

        kid = str(random.randrange(100000000, 999999999))



# -*- coding: utf-8 -*-
"""
Django settings for Medlemssys project.

For more information on this file, see
https://docs.djangoproject.com/en/dev/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/dev/ref/settings/
"""
from __future__ import absolute_import, unicode_literals

import os


# (./medlemssys/config/settings/__init__.py => ./)
APPS_DIR = os.path.normpath(os.path.dirname(__file__) + '/../../')
ROOT_DIR = os.path.normpath(APPS_DIR + '/../')


# APP CONFIGURATION
# ------------------------------------------------------------------------------
DJANGO_APPS = (
    # Default Django apps:
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    # Useful template tags:
    'django.contrib.humanize',

    # Admin
    'django.contrib.admin',
)
THIRD_PARTY_APPS = (
    'rest_framework',  # API
    'rest_framework.authtoken',
    'reversion',  # History
    'reversion_compare',  # UI for history
)

# Apps specific for this project go here.
LOCAL_APPS = (
    'medlemssys.medlem',
    'medlemssys.innhenting',
    'medlemssys.statistikk',
    'medlemssys.giro',
    'medlemssys.api',
)

# See: https://docs.djangoproject.com/en/dev/ref/settings/#installed-apps
INSTALLED_APPS = DJANGO_APPS + THIRD_PARTY_APPS + LOCAL_APPS

# MIDDLEWARE CONFIGURATION
# ------------------------------------------------------------------------------
MIDDLEWARE_CLASSES = (
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

# MIGRATIONS CONFIGURATION
# ------------------------------------------------------------------------------
#MIGRATION_MODULES = {
#    'sites': 'medlemssys.contrib.sites.migrations'
#}

# DEBUG
# ------------------------------------------------------------------------------
# See: https://docs.djangoproject.com/en/dev/ref/settings/#debug
DEBUG = False

# FIXTURE CONFIGURATION
# ------------------------------------------------------------------------------
# See: https://docs.djangoproject.com/en/dev/ref/settings/#std:setting-FIXTURE_DIRS
FIXTURE_DIRS = (
    os.path.join(APPS_DIR, 'fixtures'),
)

# EMAIL CONFIGURATION
# ------------------------------------------------------------------------------
EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'

# MANAGER CONFIGURATION
# ------------------------------------------------------------------------------
# See: https://docs.djangoproject.com/en/dev/ref/settings/#admins
ADMINS = (
    ("""Odin Hørthe Omdal""", 'odin.omdal+medlemssys@gmail.com'),
)

# See: https://docs.djangoproject.com/en/dev/ref/settings/#managers
MANAGERS = ADMINS

# DATABASE CONFIGURATION
# ------------------------------------------------------------------------------
# See: https://docs.djangoproject.com/en/dev/ref/settings/#databases
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql',
        'NAME': 'medlemssys',
    }
}


# GENERAL CONFIGURATION
# ------------------------------------------------------------------------------
# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# In a Windows environment this must be set to your system time zone.
TIME_ZONE = 'UTC'

# See: https://docs.djangoproject.com/en/dev/ref/settings/#language-code
LANGUAGE_CODE = 'nn-no'

# See: https://docs.djangoproject.com/en/dev/ref/settings/#site-id
SITE_ID = 1

# See: https://docs.djangoproject.com/en/dev/ref/settings/#use-i18n
USE_I18N = True

# See: https://docs.djangoproject.com/en/dev/ref/settings/#use-l10n
USE_L10N = True

# See: https://docs.djangoproject.com/en/dev/ref/settings/#use-tz
USE_TZ = True

# TEMPLATE CONFIGURATION
# ------------------------------------------------------------------------------
# See: https://docs.djangoproject.com/en/dev/ref/settings/#templates
TEMPLATES = [
    {
        # See: https://docs.djangoproject.com/en/dev/ref/settings/#std:setting-TEMPLATES-BACKEND
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-dirs
        'DIRS': [
            os.path.join(APPS_DIR, 'templates'),
        ],
        'OPTIONS': {
            # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-debug
            'debug': DEBUG,
            # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-loaders
            # https://docs.djangoproject.com/en/dev/ref/templates/api/#loader-types
            'loaders': [
                'django.template.loaders.filesystem.Loader',
                'django.template.loaders.app_directories.Loader',
            ],
            # See: https://docs.djangoproject.com/en/dev/ref/settings/#template-context-processors
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

# STATIC FILE CONFIGURATION
# ------------------------------------------------------------------------------
# See: https://docs.djangoproject.com/en/dev/ref/settings/#static-root
STATIC_ROOT = os.path.join(ROOT_DIR, 'staticfiles')

# See: https://docs.djangoproject.com/en/dev/ref/settings/#static-url
STATIC_URL = '/static/'

# See: https://docs.djangoproject.com/en/dev/ref/contrib/staticfiles/#std:setting-STATICFILES_DIRS
STATICFILES_DIRS = (
    os.path.join(APPS_DIR, 'static'),
)

# See: https://docs.djangoproject.com/en/dev/ref/contrib/staticfiles/#staticfiles-finders
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

# MEDIA CONFIGURATION
# ------------------------------------------------------------------------------
# See: https://docs.djangoproject.com/en/dev/ref/settings/#media-root
MEDIA_ROOT = os.path.join(APPS_DIR, 'media')

# See: https://docs.djangoproject.com/en/dev/ref/settings/#media-url
MEDIA_URL = '/media/'

# URL Configuration
# ------------------------------------------------------------------------------
ROOT_URLCONF = 'medlemssys.config.urls'

# See: https://docs.djangoproject.com/en/dev/ref/settings/#wsgi-application
WSGI_APPLICATION = 'medlemssys.config.wsgi.application'

# Location of root django.contrib.admin URL, use {% url 'admin:index' %}
ADMIN_URL = r'^admin/'

# AUTHENTICATION CONFIGURATION
# ------------------------------------------------------------------------------
AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
)

# SLUGLIFIER
AUTOSLUG_SLUGIFY_FUNCTION = 'slugify.slugify'

# REST FRAMEWORK CONFIGURATION
# ------------------------------------------------------------------------------
REST_FRAMEWORK = {
    'DEFAULT_PERMISSION_CLASSES': (
        'rest_framework.permissions.IsAdminUser',
    ),
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.TokenAuthentication',
        'rest_framework.authentication.SessionAuthentication',
        'rest_framework.authentication.BasicAuthentication',
    ),
   'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.LimitOffsetPagination',
   'PAGE_SIZE': 200,
}

# MEDLEMSSYS CONFIGURATION
# ------------------------------------------------------------------------------
BEHAVIOUR_MODULE = 'medlemssys.behaviour.barnogungdom'
KONTONUMMER = '3450 65 48618'
ORGNUMMER = '959 358 451'
ORG_ADR = ''

# Token used for creating medlems via API. To stop stupid bots.
INNMELDING_TOKEN = ''
VERVETOPP = None

# Used for allowing CORS, and where the ?next= redirector can point
TRUSTED_EXTERNAL_DOMAINS = ['example.com']

# Deprecated
GIRO_CSV = os.path.join(ROOT_DIR, 'nmu-bet.csv')
LAG_CSV = os.path.join(ROOT_DIR, 'nmu-lag.csv')
MEDLEM_CSV = os.path.join(ROOT_DIR, 'nmu-medl.csv')



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
import datetime
import re

from django.test import TestCase
from django.utils import timezone

from medlemssys.medlem.tests import lagTestMedlemar
from medlemssys.medlem.models import LokallagOvervaking, Lokallag
from .views import _get_overvakingar
from .views import create_overvaking_email
from .views import update_lokallagstat


class OvervakingGenerationTest(TestCase):
    def setUp(self):
        self.lokallag = Lokallag.objects.create(namn='Lokallag')
        self.medlemar = lagTestMedlemar()
        weeks_ago = timezone.now() - datetime.timedelta(days=14)
        LokallagOvervaking.objects.create(
            epost='test@s0.no', lokallag=self.lokallag)
        LokallagOvervaking.objects.update(sist_oppdatert=weeks_ago)
        self.overvaking = LokallagOvervaking.objects.get()
        self.medlemar['12-betalt'].oppretta = weeks_ago
        self.medlemar['12-betalt'].lokallag = self.lokallag
        self.medlemar['12-betalt'].save()
        update_lokallagstat(weeks_ago)

    def test_get_overvakingar(self):
        self.medlemar['25'].lokallag = self.lokallag
        self.medlemar['25'].save()
        ctx_list = []
        epost_seq, overvak, sist_oppdatering, ctx = next(_get_overvakingar())
        keys = ['vekkflytta_medlem', 'endra_medlem', 'ukjend_endring',
                'nye_medlem', 'utmeld_medlem', 'tilflytta_medlem',
                'nye_infofolk']
        self.assertEquals(ctx.keys(), keys)
        self.assertEquals([len(ctx[k]) for k in keys], [0, 0, 0, 1, 0, 0, 0])
        self.assertEquals(ctx['nye_medlem'][0].fornamn, '25')


class OvervakingEpostTest(TestCase):
    def setUp(self):
        self.medlemar = lagTestMedlemar()

    def test_nye_medlem(self):
        m12 = self.medlemar["12"]
        m25b = self.medlemar["25-betalt"]
        msg = create_overvaking_email(
                ['test@s0.no'],
                'Testovervaking',
                self.medlemar.values()[0].lokallag,
                sist_oppdatering=timezone.now() -
                    datetime.timedelta(days=13),
                nye_medlem=(m12, m25b))
        txt = msg.alternatives[0][0]
        re_epost = (r'.*To nye medlemar.*'
            r'<td>{m12} <td>{m12.fodt} <td>Nei .*'
            r'<td>{m25b} <td>{m25b.fodt} <td>Ja .*'
            ''.format(m12=m12, m25b=m25b))
        self.assertRegexpMatches(txt, re.compile(re_epost, re.DOTALL))

    def test_endra_medlem(self):
        m12 = self.medlemar["12"]
        m12.changed = [('test', '1', '2'), ('thing', 'a', 'b')]
        m25b = self.medlemar["25-betalt"]
        msg = create_overvaking_email(
                ['test@s0.no'],
                'Testovervaking',
                self.medlemar.values()[0].lokallag,
                sist_oppdatering=timezone.now() - datetime.timedelta(days=13),
                endra_medlem=(m12, m25b))
        txt = msg.alternatives[0][0]
        re_epost = (
            r'.*To medlemar med endringar.*'
            r'<li><strong>{m12}</strong>.*'
            r'<th.*>test <td>1 <td>2.*'
            r'<th.*>thing <td>a <td>b.*'
            r'<li><strong>{m25b}</strong>.*'
            ''.format(m12=m12, m25b=m25b))
        self.assertRegexpMatches(txt, re.compile(re_epost, re.DOTALL))

    def test_utmeld_medlem(self):
        m12 = self.medlemar["12-utmeld"]
        m25b = self.medlemar["25-betalt-utmeld"]
        msg = create_overvaking_email(
                ['test@s0.no'],
                'Testovervaking',
                self.medlemar.values()[0].lokallag,
                sist_oppdatering=timezone.now() - datetime.timedelta(days=13),
                utmeld_medlem=(m12, m25b))
        txt = msg.alternatives[0][0]
        re_epost = (r'.*To utmelde medlemar.*'
            r'<td>{m12} <td>{m12.fodt} <td>Nei .*'
            r'<td>{m25b} <td>{m25b.fodt} <td>Ja .*'
            ''.format(m12=m12, m25b=m25b))
        self.assertRegexpMatches(txt, re.compile(re_epost, re.DOTALL))

    def test_tilflytta_medlem(self):
        m12 = self.medlemar["12"]
        m25b = self.medlemar["25-betalt"]
        msg = create_overvaking_email(
                ['test@s0.no'],
                'Testovervaking',
                self.medlemar.values()[0].lokallag,
                sist_oppdatering=timezone.now() - datetime.timedelta(days=13),
                tilflytta_medlem=(m12, m25b))
        txt = msg.alternatives[0][0]
        re_epost = (r'.*To nytilflytta medlemar.*'
            r'<td>{m12} <td>{m12.fodt} <td>Nei .*'
            r'<td>{m25b} <td>{m25b.fodt} <td>Ja .*'
            ''.format(m12=m12, m25b=m25b))
        self.assertRegexpMatches(txt, re.compile(re_epost, re.DOTALL))

    def test_vekkflytta_medlem(self):
        m12 = self.medlemar["12"]
        m25b = self.medlemar["25-betalt"]
        msg = create_overvaking_email(
                ['test@s0.no'],
                'Testovervaking',
                self.medlemar.values()[0].lokallag,
                sist_oppdatering=timezone.now() - datetime.timedelta(days=13),
                vekkflytta_medlem=(m12, m25b))
        txt = msg.alternatives[0][0]
        re_epost = (ur'.*To fråflytta medlemar.*'
            r'<td>{m12} <td>{m12.fodt} <td>Nei .*'
            r'<td>{m25b} <td>{m25b.fodt} <td>Ja .*'
            ''.format(m12=m12, m25b=m25b))
        self.assertRegexpMatches(txt, re.compile(re_epost, re.DOTALL))



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _

from medlemssys.medlem.models import Lokallag


class LokallagStat(models.Model):
    lokallag = models.ForeignKey(Lokallag, null=True, on_delete=models.SET_NULL)
    veke = models.CharField(_("År-veke"), max_length=7)

    n_teljande = models.IntegerField(_("tal på teljande"))
    n_interessante = models.IntegerField(_("tal på interessante"))
    n_ikkje_utmelde = models.IntegerField(_("tal på ikkje utmelde"))
    n_totalt = models.IntegerField(_("tal på medlemar registrert"))

    teljande = models.TextField(_("liste over teljande"))
    interessante = models.TextField(_("liste over interessante"))

    oppretta = models.DateTimeField(_("oppretta"), auto_now_add=True)

    class Meta:
        unique_together = ('lokallag', 'veke')

    def __unicode__(self):
        return "{0} {1}".format(self.lokallag, self.veke)



# -*- coding: utf-8 -*-
# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
from django import forms
from models import Giro

class GiroForm(forms.ModelForm):
    class Meta:
        model = Giro
        fields = ('belop', 'konto', 'desc', 'hensikt')

class SuggestedLokallagForm(forms.Form):
    lokallag = forms.ChoiceField(choices=[('', '---')],
                                 widget=forms.RadioSelect())

    class Meta:
        fields = ('lokallag',)

    def __init__(self, *args, **kwargs):
        choices = kwargs.pop('lokallag_choices')
        medlem_id = kwargs.pop('medlem_id')
        new_name = 'lokallag_{}'.format(medlem_id)
        super(type(self), self).__init__(*args, **kwargs)
        self.initial[new_name] = self.initial.pop('lokallag')
        self.fields[new_name] = self.fields.pop('lokallag')
        self.fields[new_name].choices = choices



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2016 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.

from datetime import date

from django.contrib.admin.filters import DateFieldListFilter
from django.contrib.admin.filters import ListFilter
from django.contrib.admin.filters import RelatedFieldListFilter
from django.contrib.admin.filters import SimpleListFilter
from django.db.models import Q
from django.utils.encoding import smart_unicode
from django.utils.translation import ugettext as _

from medlemssys.behaviour import get_behaviour

from .models import PostNummer


class CustomFilter(SimpleListFilter):
    behaviour_variable = None

    def __init__(self, *args, **kwargs):
        self.filters = getattr(get_behaviour(), self.behaviour_variable)
        return super(CustomFilter, self).__init__(*args, **kwargs)

    def lookups(self, request, model_admin):
        return [(v.key, v.title) for v in self.filters]

    def queryset(self, request, queryset):
       lookup = next((v for v in self.filters if v.key == self.value()), None)
       if lookup:
           return lookup.filter(queryset)


class SporjingFilter(CustomFilter):
    title = _(u"spørjingar")
    parameter_name = 'sporjing'
    behaviour_variable = 'medlem_ui_filters'


class GiroSporjingFilter(CustomFilter):
    title = _(u"giro-spørjingar")
    parameter_name = 'sporjing'
    behaviour_variable = 'giro_ui_filters'


class StadFilter(ListFilter):
    title = _(u"stad")

    def __init__(self, request, params, model, model_admin):
        super(StadFilter, self).__init__(
            request, params, model, model_admin)
        self.request = request
        for p in self.expected_parameters():
            if p in params:
                value = params.pop(p)
                self.used_parameters[p] = value

    def param(self, var):
        return self.used_parameters.get(var, None)

    def has_output(self):
        return True

    def expected_parameters(self):
        return ('stad_fylke', 'stad_kommune')

    def choices(self, cl):
        qs = cl.get_queryset(self.request)
        yield {
            'selected': self.param('stad_fylke') is None,
            'query_string': cl.get_query_string(
                {}, ['stad_fylke', 'stad_kommune']),
            'display': _('All'),
        }
        if self.param('stad_fylke') is None:
            if qs.count() < 1000:
                # Converting the QuerySet __in to a list because of a MySQL
                # performance issue
                fylke_qs = PostNummer.objects.filter(
                    postnr__in=list(qs.values('postnr')))
            else:
                fylke_qs = PostNummer.objects.all()
            for fylke in fylke_qs.values_list('fylke', flat=True).distinct():
                yield {
                    'selected': self.param('stad_fylke') == fylke,
                    'query_string': cl.get_query_string({
                        'stad_fylke': fylke,
                    }, []),
                    'display': fylke,
                }
        else:
            fylke = self.param('stad_fylke')
            yield {
                'selected': True,
                'query_string': cl.get_query_string({
                    'stad_fylke': fylke,
                }, ['stad_kommune']),
                'display': fylke,
            }
            # Kommunar som er i dette fylket

            # Converting the QuerySet __in to a list because of a MySQL
            # performance issue
            kommune_qs = (
                PostNummer.objects.filter(
                    postnr__in=list(qs.values_list('postnr', flat=True)),
                    fylke=fylke)
                .values_list('kommune', flat=True)
                .distinct())
            for kommune in kommune_qs:
                yield {
                    'selected': self.param('stad_kommune') == kommune,
                    'query_string': cl.get_query_string({
                        'stad_kommune': kommune,
                    }, []),
                    'display': u'– {0}'.format(smart_unicode(kommune)),
                }

    def queryset(self, request, queryset):
        if self.param('stad_kommune'):
            return queryset.kommune(
                       self.param('stad_kommune'),
                       fylke=self.param('stad_fylke'))
        elif self.param('stad_fylke'):
            return queryset.fylke(self.param('stad_fylke'))
        return queryset


class FodtFilter(SimpleListFilter):
    title = _(u"alder i år")
    parameter_name = "alder"
    field = 'fodt'

    def lookups(self, request, model_admin):
        return (
                ('under-26', u"Under 26 (teljande)"),
                ('over-25',  u"26 og eldre"),
                ('under-30', u"Under 30"),
                ('over-29',  u"30 og eldre"),
                ('25',       u"25 år (sisteårs teljande)"),
                ('26',       u"26 år (ikkje teljande fyrste år)"),
                ('invalid',  u"Sære aldre"),
            )

    def queryset(self, request, queryset):
        year = date.today().year

        restriction = {}
        if self.value() == 'under-26':
            restriction = { self.field + '__gt': (year - 26)}
        elif self.value() == 'over-25':
            restriction = { self.field + '__lt': (year - 25)}
        if self.value() == 'under-30':
            restriction = { self.field + '__gt': (year - 30)}
        elif self.value() == 'over-29':
            restriction = { self.field + '__lt': (year - 29)}
        elif self.value() == '25':
            restriction = { self.field: (year - 25)}
        elif self.value() == '26':
            restriction = { self.field: (year - 26)}
        elif self.value() == 'invalid':
            return queryset.filter(Q(**{self.field + '__isnull': True}) |
                                   Q(**{self.field + '__lt': (year-120)}) |
                                   Q(**{self.field + '__gt': year}))
        return queryset.filter(**restriction)


class MedlemFodtFilter(FodtFilter):
    field = 'medlem__fodt'


class TimeSinceFilter(DateFieldListFilter):
    def __init__(self, field, request, params, model, model_admin, field_path):
        super(TimeSinceFilter, self).__init__(field, request, params,
                model, model_admin, field_path)

        this_year = date(date.today().year, 1, 1)
        last_year = date(date.today().year - 1, 1, 1)

        self.lookup_kwarg_past_time = '%s__gte' % self.field_path
        self.lookup_kwarg_lt_time = '%s__lt' % self.field_path
        self.lookup_kwarg_isnull = '%s__isnull' % self.field_path

        self.links = (
            (_('All'), {}),
            (_(smart_unicode('Innan året')), {
                self.lookup_kwarg_past_time: str(this_year),
            }),
            (_(smart_unicode('Innan fjoråret')), {
                self.lookup_kwarg_past_time: str(last_year),
            }),
            (_(smart_unicode('I fjor')), {
                self.lookup_kwarg_past_time: str(last_year),
                self.lookup_kwarg_lt_time: str(this_year),
            }),
            (_(smart_unicode('I fjor eller eldre')), {
                self.lookup_kwarg_lt_time: str(this_year),
            }),
            (_('Aldri'), {
                self.lookup_kwarg_isnull: str(True),
            }),
            (_('Har skjedd'), {
                self.lookup_kwarg_isnull: str(False),
            }),
        )


class AdditiveSubtractiveFilter(RelatedFieldListFilter):
    def __init__(self, field, request, params, model, model_admin, field_path):
        self.using_params = []
        self.paramstart = "adv_" + field.get_attname()

        super(AdditiveSubtractiveFilter, self).__init__(
            field, request, params, model, model_admin, field_path)

    def has_output(self):
        return len(self.lookup_choices) > 0

    def _make_param(self, field_id):
        return self.paramstart + str(field_id)

    def expected_parameters(self):
        for lookup, title in self.lookup_choices:
            self.using_params.append(self._make_param(lookup))
        return self.using_params

    def queryset(self, request, queryset):
        for key, value in self.used_parameters.items():
            if value == 'exc':
                queryset = queryset.exclude(
                    val__id=key.replace(self.paramstart, ''))
            elif value == 'inc':
                queryset = queryset.filter(
                    val__id=key.replace(self.paramstart, ''))
        return queryset

    def choices(self, cl):
        used_params = set(self.used_parameters).intersection(self.using_params)
        yield {
            'selected': len(used_params) == 0,
            'query_string': cl.get_query_string({}, self.using_params),
            'display': _("All"),
        }
        for p in (('inc', _("Vis")), ('exc', _("Skjul"))):
            for lookup, title in self.lookup_choices:
                # If the link is selected
                if p[0] == self.used_parameters.get(self._make_param(lookup)):
                    yield {
                        'selected': True,
                        'query_string': cl.get_query_string({},
                            [ self._make_param(lookup) ]),
                        'display': "%s '%s'" % (p[1], title),
                    }
                else:
                    yield {
                        'selected': False,
                        'query_string': cl.get_query_string({
                            self._make_param(lookup): p[0],
                        }, []),
                        'display': "%s '%s'" % (p[1], title),
                    }



# -*- coding: utf-8 -*-
# vim: shiftwidth=4 tabstop=4 expandtab softtabstop=4 ai

# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import print_function

import os

from django.conf import settings
from django.core.management.base import BaseCommand
from django.core.management.base import CommandError
from django.db import transaction
from django.db.models import Q

from medlemssys.medlem.models import Giro
from medlemssys.medlem.models import update_denormalized_fields
from medlemssys.ocr import parse_ocr

obj = ""

class Command(BaseCommand):
    args = '[ ocr_fil.txt ]'
    help = 'Registrerer OCR-innbetalingar'

    @transaction.atomic
    def handle(self, *args, **options):
        if len(args):
            fn = args[0]
        else:
            fn = getattr(settings, 'OCR_FIL', 'ocr.txt')

        if not os.path.isfile(fn):
            raise CommandError("Fila finst ikkje ({0})".format(fn).encode('utf8'))

        ocr_file = open(fn)
        ocr = parse_ocr(ocr_file)

        for f in ocr:
            try:
                giro = Giro.objects.get(
                        Q(oppretta__year=f['dato'].year) | Q(oppretta__year=(f['dato'].year - 1)),
                        kid=f['kid'],
                    )
            except Giro.DoesNotExist:
                self.err("Fann ikkje giroen, {dato:6s} {belop:4n} {kid:12s} {transaksjon}".format(**f))
                continue

            if f['belop'] < giro.belop:
                self.err("{giro}: for lite betalt! Rekna {giro.belop}, fekk {belop} ({giro.pk})".format(giro=giro, belop=f['belop']))
            elif f['belop'] > giro.belop:
                # XXX: Splitt opp? Registrer ein donasjon?
                self.err("{giro}: Betalte meir, venta {giro.belop}, fekk {belop} ({giro.pk})".format(giro=giro, belop=f['belop']))
                giro.innbetalt = f['dato']
            else:
                giro.innbetalt = f['dato']
                #self.err("{belop:3n}kr ({giro.pk}) {giro} ".format(giro=giro, belop=f['belop']))

            giro.innbetalt_belop = f['belop']
            giro.status = 'F'
            if giro.desc:
                giro.desc += '\nOCR'
            else:
                giro.desc = 'OCR'
            giro.save()
            self.err(u"{giro}: Betalte {giro.belop} ({giro.pk})".format(giro=giro))

            if giro.medlem.status == 'I' and giro.innbetalt_belop >= giro.belop:
                self.err("{medlem} var infoperson, flyttar til medlem etter betaling".format(medlem=giro.medlem))
                giro.medlem.status = 'M'
                giro.medlem.save()

        update_denormalized_fields()


    def err(self, msg):
        self.stderr.write((unicode(msg) + "\n").encode('utf-8'))



# -*- coding: utf-8 -*-
"""
Local settings

- Run in Debug mode
- Use console backend for emails
- Add Django Debug Toolbar
- Add django-extensions as app
"""

from .common import *  # noqa

VERVETOPP = True

# DEBUG
# ------------------------------------------------------------------------------
DEBUG = True
TEMPLATES[0]['OPTIONS']['debug'] = DEBUG
TEMPLATES[0]['OPTIONS']['string_if_invalid'] = 'INVALID EXPRESSION: %s'

# SECRET CONFIGURATION
# ------------------------------------------------------------------------------
# See: https://docs.djangoproject.com/en/dev/ref/settings/#secret-key
# Note: This key only used for development and testing.
SECRET_KEY = 'ENDRA_MEG-e!(o%l1myqy-v(ocxf*xkr)q#=l-^%yxgcod_uicne1wh5ggi1'

# MAIL SETTINGS
# ------------------------------------------------------------------------------

EMAIL_PORT = 1025

EMAIL_HOST = 'localhost'
EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'


# CACHING
# ------------------------------------------------------------------------------
CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'LOCATION': ''
    }
}

# django-debug-toolbar
# ------------------------------------------------------------------------------
MIDDLEWARE_CLASSES += ('debug_toolbar.middleware.DebugToolbarMiddleware',)
INSTALLED_APPS += ('debug_toolbar', )

INTERNAL_IPS = ('127.0.0.1', '10.0.2.2',)

DEBUG_TOOLBAR_CONFIG = {
    'DISABLE_PANELS': [
        'debug_toolbar.panels.redirects.RedirectsPanel',
    ],
    'SHOW_TEMPLATE_CONTEXT': True,
}

# django-extensions
# ------------------------------------------------------------------------------
INSTALLED_APPS += ('django_extensions', )

# TESTING
# ------------------------------------------------------------------------------
TEST_RUNNER = 'django.test.runner.DiscoverRunner'



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
import logging
from django.shortcuts import render
from .ocr import OCR, OCRError

logger = logging.getLogger(__name__)


def import_ocr(request):
    errorlist = []
    if request.method == 'POST':
        ocr_data = request.POST.get('ocr_data', '')
        ocr = OCR()
        logger.debug("Running OCR file: {}".format(ocr_data))
        try:
            ocr.from_string(ocr_data)
        except OCRError as e:
            errorlist.append("Feil med OCR-fil: {}".format(unicode(e)))
        else:
            ocr.process_to_db()
            processed = filter(lambda o: o['processed'], ocr.data)
            not_processed = filter(lambda o: not o['processed'], ocr.data)
            return render(request, 'admin/import_ocr_done.htm', {
                    'not_processed': not_processed,
                    'processed': processed,
                    'title': u'OCR-køyring',
                })

    return render(request, 'admin/import_ocr.htm', {
        'title': u'OCR-køyring',
        'errorlist': errorlist,
    })
